#!/usr/bin/env python3
# -*- coding: utf-8 -*-
'''
Criticality: Low
Check that the monitoring graphs work.
'''
from datetime import datetime
from pathlib import Path
import os
import subprocess
import sys

sys.path.append(str(Path(__file__).parents[1].resolve()))

# pylint: disable=wrong-import-position
from utilities import logging as lg  # noqa: E402


MUNIN_WWW_PATH = '/var/cache/munin/www/'


def check_munin_node():
    lg.log('Checking if Munin node works...')

    # check if package is installed
    p = subprocess.run(['dpkg', '-s', 'munin-node'], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    if p.returncode != 0:
        lg.info('The command "dpkg -s munin-node" returned code %s, assuming that the package is not installed. Test skipped.' % p.returncode)
        return -1

    # check that no plugin should be activated or disabled
    lg.log('Checking that no plugin should be enabled/disabled.')
    lg.log('munin-node-configure --shell --remove-also  # stderr is hidden')
    p = subprocess.run(['munin-node-configure', '--shell', '--remove-also'], stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
    out = p.stdout.decode('utf-8').strip()
    lg.log(out)
    if out:
        to_be_enabled_count = out.count('ln -s')
        if to_be_enabled_count:
            lg.warning('%s plugins should be enabled.' % to_be_enabled_count)
        to_be_disabled_count = out.count('rm -f')
        if to_be_disabled_count:
            lg.warning('%s plugins should be disabled.' % to_be_disabled_count)
        lg.log('''To enable/disable correct plugins, please run this command:
munin-node-configure --shell --remove-also 2>/dev/null | sh; systemctl restart munin 2>/dev/null; systemctl restart munin-node;
''')
    return 0


def check_munin():
    lg.log('Checking if Munin graph works...')

    # check if package is installed
    p = subprocess.run(['dpkg', '-s', 'munin'], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    if p.returncode != 0:
        lg.info('The command "dpkg -s munin" returned code %s, assuming that the package is not installed. Test skipped.' % p.returncode)
        return -1

    # get Munin www dir names
    if os.path.exists(MUNIN_WWW_PATH):
        names = os.listdir(MUNIN_WWW_PATH)
        names.sort()
    else:
        names = []
    if not names:
        lg.error('No Munin directory found in "%s".' % MUNIN_WWW_PATH)
        return 1

    # get cpu day graph of each host
    paths = list()
    for name in names:
        if not name.endswith('.html') and name != 'static' and os.path.isfile(os.path.join(MUNIN_WWW_PATH, name, 'index.html')):
            for sub_name in os.listdir(os.path.join(MUNIN_WWW_PATH, name)):
                path = os.path.join(MUNIN_WWW_PATH, name, sub_name, 'cpu-day.png')
                if sub_name != 'index.html' and os.path.exists(path):
                    paths.append(path)
    if not paths:
        lg.error('No Munin host directory was found in "%s".' % MUNIN_WWW_PATH)
        return 1

    # check graph mtime
    error = False
    for path in paths:
        lg.log('Checking graph "%s" modification date...' % path)
        mtime = os.path.getmtime(path)
        d = datetime.fromtimestamp(mtime)
        now = datetime.now()
        diff_seconds = (now - d).total_seconds()
        if diff_seconds > 3600:
            lg.error('The graph is older than 1 hour. The monitoring is probably not working.')
            error = True
        else:
            lg.success('The graph is not older than 1 hour.')
    return 1 if error else 0


if __name__ == '__main__':
    code1 = check_munin_node()
    code2 = check_munin()
    sys.exit(max(code1, code2))
